#include <stdio.h>
#include <stdlib.h>
#include <SDL/SDL.h>
#include "types.h"
#include "polygon.h"

/* Draws the polygon described by the point list PointList in colour */
/* Colour with all vertices offset by (X,Y) */
#define DRAW_POLYGON(PointList, Colour, X, Y)                  \
    Polygon.Length = sizeof(PointList) / sizeof(struct Point); \
    Polygon.PointPtr = PointList;                              \
    FillConvexPolygon(&Polygon, Colour, X, Y);

extern S32 FillConvexPolygon(struct PointListHeader *VertexList ,S32 Colour,
                                 S32 XOffset, S32 YOffset);
                                 
extern U8 InitSDLScreenBuf(void);
extern void redraw(void);
extern void cls(void);

/* simple program to exercise the polygo filling routines */
int main(int argc, char *argv[])
{
    S32 i;
    S32 j;
    input_type UserControls;
    memset(&UserControls,0,sizeof(UserControls));
    poly_type SelectPoly = poly0;

    struct PointListHeader Polygon;
    static struct Point ScreenRectangle[] = {{0,0},{320,0},{320,200},{0,200}};
    static struct Point ConvexShape[] =
           {{0,0},{121,0},{320,0},{200,51},{301,51},{250,51},{319,143},
            {320,200},{22,200},{0,200},{50,180},{20,160},{50,140},
            {20,120},{50,100},{20,80},{50,60},{20,40},{50,20}};
    static struct Point Hexagon[] =
           {{90,-50},{0,-90},{-90,-50},{-90,50},{0,90},{90,50}};
    static struct Point Triangle1[] = {{30,0},{15,20},{0,0}};
    static struct Point Triangle2[] = {{30,20},{15,0},{0,20}};
    static struct Point Triangle3[] = {{0,20},{20,10},{0,0}};
    static struct Point Triangle4[] = {{20,20},{20,0},{0,10}};
    
    /* try and get a handle to the screen */
    if(InitSDLScreenBuf() == 1)
    {
        /* cant get handle to screen so exit */
        return(1);
    }
    
    /* clear screen to cyan */
    DRAW_POLYGON(ScreenRectangle, 3, 0, 0);
    SelectPoly++;
    redraw();
    
    /* draw an irregular shape that meets our definition of a convex but
    is not convex by any normal description */
    DRAW_POLYGON(ConvexShape, 6, 0, 0);
    SelectPoly++;
    redraw();
    
    /* loop round until user exits */
    while(UserControls.Exit == 0)
    {
        /* clear last user input */
        memset(&UserControls,0,sizeof(UserControls));
        
        /* delay getting user input by approx 100ms */
        SDL_Delay(100);
        
        /* then see if user was pressed a key */
        GetUserInput(&UserControls);
        
        /* check to see if user has pressed the up button */
        if(UserControls.Forward == 1)
        {
            switch(SelectPoly)
            {   
                case poly2:
                      /* Draw adjacent triangles across the top half of the screen */
                     for (j=0; j<=80; j+=20)
                     {
                         for (i=0; i<290; i += 30)
                         {
                             DRAW_POLYGON(Triangle1, 2, i, j);
                             DRAW_POLYGON(Triangle2, 4, i+15, j);
                         }
                     }
                     redraw();
                     SelectPoly++;
                break;
                
                case poly3:
                    /* Draw adjacent triangles across the bottom half of the screen */
                    for (j=100; j<=170; j+=20)
                    {
                        /* Do a row of pointing-right triangles */
                        for (i=0; i<290; i += 20)
                        {
                            DRAW_POLYGON(Triangle3, 40, i, j);
                        }
                        /* Do a row of pointing-left triangles halfway between one row
                        of pointing-right triangles and the next, to fit between */
                        for (i=0; i<290; i += 20)
                        {
                            DRAW_POLYGON(Triangle4, 1, i, j+10);
                        }
                     }
                     redraw();
                     SelectPoly++;
                break;
                
                case poly4:
                    /* Finally, draw a series of concentric hexagons of approximately
                    the same proportions in the center of the screen */
                    for (i=0; i < 8; i++)
                    {
                        DRAW_POLYGON(Hexagon, i, 160, 100);
                        redraw();
                        SDL_Delay(50);
                        for (j=0; j<sizeof(Hexagon)/sizeof(struct Point); j++) 
                        {
                            /* Advance each vertex toward the center */
                            if (Hexagon[j].X != 0)
                            {
                                Hexagon[j].X -= Hexagon[j].X >= 0 ? 3 : -3;
                                Hexagon[j].Y -= Hexagon[j].Y >= 0 ? 2 : -2;
                            } 
                            else 
                            {
                                Hexagon[j].Y -= Hexagon[j].Y >= 0 ? 3 : -3;
                            }
                        }
                    }                  
                    SelectPoly++;
                break;     
                
                default:
                    /* do nothing */
                break;        
            }
        }
    }

    return 0;
}
